#include <iostream>
#include <string>
#include <vector>
#include <functional>

class Observable;

class Observer {
public:
	virtual ~Observer() {}
	virtual void Update(const Observable& obs) = 0;
};

class Observable {
public:
	void RegisterObserver(Observer* observer) {
		observers_.push_back(observer);
	}

	void NotifyObservers() const {
		for (const auto& observer : observers_) {
			observer->Update(*this);
		}
	}

private:
	std::vector<Observer*> observers_;
};

class Product : public Observable {
public:
	Product(const std::string& name, double price)
		: name_(name), price_(price) {}

	void SetPrice(double price) {
		price_ = price;
		NotifyObservers();
	}

	std::string GetName() const { return name_; }
	double GetPrice() const { return price_; }

private:
	std::string name_;
	double price_;
};

class Subscriber : public Observer {
public:
	Subscriber(const std::string& name, Product& product)
		: name_(name), product_(product) {
		product_.RegisterObserver(this);
	}

	void Update(const Observable& obs) override {
		std::cout << name_ << ": Product " << product_.GetName()
			<< " price changed to " << product_.GetPrice() << std::endl;
	}

private:
	std::string name_;
	Product& product_;
};

int main() {
	Product laptop("Laptop", 1000.0);
	Subscriber subscriber1("Subscriber 1", laptop);
	Subscriber subscriber2("Subscriber 2", laptop);

	laptop.SetPrice(900.0);
	laptop.SetPrice(950.0);
	return 0;
}
