#include <iostream>
#include <vector>
#include <algorithm>

// ۲
class Observer {
public:
	virtual void Update(int value) = 0;
};

// ۲
class ConcreteObserver : public Observer {
public:
	ConcreteObserver(int id) : id_(id) {}

	void Update(int value) override {
		std::cout << "Observer " << id_ << " received update: " << value << std::endl;
	}

private:
	int id_;
};

// 
class Subject {
public:
	virtual void RegisterObserver(Observer* observer) = 0;
	virtual void RemoveObserver(Observer* observer) = 0;
	virtual void NotifyObservers() = 0;
};

// 
class ConcreteSubject : public Subject {
public:
	void RegisterObserver(Observer* observer) override {
		observers_.push_back(observer);
	}

	void RemoveObserver(Observer* observer) override {
		auto it = std::find(observers_.begin(), observers_.end(), observer);
		if (it != observers_.end()) {
			observers_.erase(it);
		}
	}

	void NotifyObservers() override {
		for (Observer* observer : observers_) {
			observer->Update(value_);
		}
	}

	void SetValue(int value) {
		value_ = value;
		NotifyObservers();
	}

private:
	std::vector<Observer*> observers_;
	int value_;
};

//int main() {
//	ConcreteSubject subject;
//	ConcreteObserver observer1(1);
//	ConcreteObserver observer2(2);
//
//	subject.RegisterObserver(&observer1);
//	subject.RegisterObserver(&observer2);
//
//	subject.SetValue(123);
//
//	subject.RemoveObserver(&observer1);
//
//	subject.SetValue(456);
//
//	return 0;
//}
