#include <iostream>
#include <vector>
#include <queue>
#include <thread>
#include <condition_variable>
#include <functional>
#include <mutex>

//class Task {
//public:
//	using TaskFunction = std::function<void()>;
//
//	explicit Task(TaskFunction taskFunction)
//		: taskFunction(std::move(taskFunction)) {}
//
//	void operator()() const {
//		taskFunction();
//	}
//
//private:
//	TaskFunction taskFunction;
//};

class Task {
public:
	Task() {};
	Task(const std::function<void()>& task) : task_(task) {}

	void operator()() const { task_(); }

private:
	std::function<void()> task_;
};

class ThreadPool {
public:
	explicit ThreadPool(std::size_t numThreads) {
		for (std::size_t i = 0; i < numThreads; ++i) {
			workers.emplace_back([this] {
				for (;;) {
					Task task;

					{
						std::unique_lock<std::mutex> lock(this->queueMutex);

						this->condition.wait(
							lock, [this] { return this->stop || !this->tasks.empty(); });

						if (this->stop && this->tasks.empty()) {
							return;
						}

						task = std::move(this->tasks.front());
						this->tasks.pop();
					}

					task();
				}
			});
		}
	}

	~ThreadPool() {
		{
			std::unique_lock<std::mutex> lock(queueMutex);
			stop = true;
		}

		condition.notify_all();
		for (std::thread &worker : workers) {
			worker.join();
		}
	}

	template <typename TaskType>
	void enqueue(TaskType task) {
		{
			std::unique_lock<std::mutex> lock(queueMutex);

			tasks.emplace(std::move(task));
		}

		condition.notify_one();
	}

private:
	std::vector<std::thread> workers;
	std::queue<Task> tasks;

	std::mutex queueMutex;
	std::condition_variable condition;
	bool stop = false;
};

class Observable {
public:
	using Observer = std::function<void(int)>;

	void registerObserver(Observer observer) {
		observers.emplace_back(observer);
	}

	void notifyObservers(int value) {
		for (const Observer &observer : observers) {
			observer(value);
		}
	}

private:
	std::vector<Observer> observers;
};

//int main() {
//	Observable observable;
//	ThreadPool threadPool(4);
//
//	observable.registerObserver([&threadPool](int value) {
//		threadPool.enqueue([value] {std::cout << "Observer received value: " << value << std::endl; });
//	});
//	observable.notifyObservers(42);
//	observable.notifyObservers(21);
//	observable.notifyObservers(69);
//	observable.notifyObservers(42);
//	observable.notifyObservers(21);
//	observable.notifyObservers(69);
//
//	return 0;
//}