#include <iostream>

class Character {
public:
	virtual int getAttackPower() = 0;
};

class Warrior : public Character {
public:
	int getAttackPower() { return 10; }
};

class CharacterDecorator : public Character {
protected:
	Character *character;

public:
	CharacterDecorator(Character *c) : character(c) {}
	int getAttackPower() { return character->getAttackPower(); }
};

class FireAttack : public CharacterDecorator {
public:
	FireAttack(Character *c) : CharacterDecorator(c) {}
	int getAttackPower() { return character->getAttackPower() + 5; }
};

class IceAttack : public CharacterDecorator {
public:
	IceAttack(Character *c) : CharacterDecorator(c) {}
	int getAttackPower() { return character->getAttackPower() - 3; }
};

int main() {
	Character *warrior = new Warrior();
	Character *fire_warrior = new FireAttack(warrior);
	Character *ice_warrior = new IceAttack(fire_warrior);

	std::cout << ice_warrior->getAttackPower() << std::endl;

	delete ice_warrior;
	return 0;
}
